@@ -7,7 +7,7 @@ from datetime import timedelta, datetime, date
 from enum import Enum, unique
 from io import StringIO
 from pathlib import Path
-from typing import List, Dict, Tuple, Callable, Union, Any, Type
+from typing import List, Dict, Tuple, Callable, Union, Any, Type, Optional
 
 import paramiko
 from paramiko.channel import Channel
@@ -68,6 +68,8 @@ def to_seconds(time_str: str) -> int:
     Returns:
         int: seconds
     """
+    if time_str == "0":
+        return 0
     for unit, sec in UNIT_SEC_PAIRS.items():
         re_time_str = r"(\d+){unit}".format(unit=unit)
         matched = re.match(re_time_str, time_str)
@@ -81,10 +83,23 @@ def humanize_seconds(seconds: int):
     return str(timedelta(seconds=seconds))
 
 
-def send_to_channel(channel: Channel, msg: str = "", prefix: str = "", suffix: str = "\r\n"):
-    if isinstance(msg, bytes):
-        msg = msg.decode()
-    channel.send(f"{prefix or ''}{msg}{suffix or ''}".encode())
+def ensure_bytes(s: Union[str, bytes]) -> bytes:
+    if isinstance(s, str):
+        return s.encode()
+    return s
+
+
+def send_to_channel(
+    channel: Channel, msg: Union[str, bytes] = b"", prefix: Union[str, bytes] = b"", suffix: Union[str, bytes] = b"\r\n"
+):
+    msg = ensure_bytes(msg)
+    prefix = ensure_bytes(prefix)
+    suffix = ensure_bytes(suffix)
+
+    buf = bytearray(prefix)
+    buf.extend(msg)
+    buf.extend(suffix)
+    channel.sendall(buf)
 
 
 def confirm_dc_type(value: Any, should_be: Type):
@@ -107,20 +122,30 @@ def confirm_dc_type(value: Any, should_be: Type):
     return value
 
 
-def choose_option(channel: Channel, options: List[str], option_prompt: str = None, ask_prompt: str = None) -> int:
+def choose_option(
+    channel: Channel,
+    options: List[str],
+    option_prompt: str = None,
+    ask_prompt: str = None,
+    default: Optional[int] = None,
+) -> int:
     """
     ask user to choose one option from channel
     """
-    logger.info(f"need user choose from {options}\nprompt: {option_prompt}")
+    logger.info(f"need user to choose from {options}\nprompt: {option_prompt}")
     send_to_channel(channel, option_prompt or "Available options:")
     for index, option in enumerate(options):
         send_to_channel(channel, f"{index:>3} - {option}")
     _ask_prompt = ask_prompt or f"Please enter the number of choice[{0}-{len(options) - 1}]: "
-    send_to_channel(channel, _ask_prompt, suffix="")
+    send_to_channel(channel, _ask_prompt, suffix=b"")
 
     result = read_user_input_line(channel)
     try:
-        num_selected = int(result)
+        if result == "" and default is not None:
+            # User hits enter directly
+            num_selected = default
+        else:
+            num_selected = int(result)
         if 0 <= num_selected < len(options):
             logger.info(f"user choose {result} for option {option_prompt}")
             send_to_channel(channel, f"You selected: {options[num_selected]}")
@@ -128,7 +153,7 @@ def choose_option(channel: Channel, options: List[str], option_prompt: str = Non
         raise Exception(f"user choose {result} for option {option_prompt}, but it's out of range")
     except Exception:  # noqa
         logger.error(f"user choose {result} for option {option_prompt} invalid, re-choose...")
-        send_to_channel(channel, f"unknown choose, please choose again [{0}-{len(options) - 1}]")
+        send_to_channel(channel, f"unknown choice, please choose again [{0}-{len(options) - 1}]")
         return choose_option(channel, options, option_prompt=option_prompt, ask_prompt=ask_prompt)
 
 
@@ -145,10 +170,10 @@ def read_user_input_line(channel) -> str:
             send_to_channel(channel)
             break
         elif content in [b"\x7f"]:
-            send_to_channel(channel, "\x08\x1b[K", suffix="")
+            send_to_channel(channel, "\x08\x1b[K", suffix=b"")
             continue
         else:
-            send_to_channel(channel, content, suffix="")
+            send_to_channel(channel, content, suffix=b"")
             chars.append(content)
     return b"".join(chars).decode()
 
@@ -158,8 +183,8 @@ class KeyTypeSupport(Enum):
     # openssh ssh-keygen: The default length is 3072 bits (RSA) or 256 bits (ECDSA).
     # todo default length of DSS/ED25519
     RSA = "RSA", 3072
-    DSS = "DSS"
-    ED25519 = "ED25519"
+    DSS = "DSS", 1024
+    ED25519 = "ED25519", 256
     ECDSA = "ECDSA", 256
 
     def __init__(self, key, default_key_length=None):
@@ -172,15 +197,17 @@ class KeyTypeSupport(Enum):
         return self._key
 
 
-def confirm_ssh_key_pair(key_type: KeyTypeSupport = KeyTypeSupport.RSA, key_len: int = None, save_path: Path = None):
-    pri_key, pub_key = try_load_key_from_file(from_path=save_path, key_type=key_type)
+def confirm_ssh_key_pair(
+    save_path: Path, key_type: KeyTypeSupport = KeyTypeSupport.RSA, key_len: int = None, key_name: str = None
+):
+    pri_key, pub_key = try_load_key_from_file(from_path=save_path, key_type=key_type, key_name=key_name)
     if pri_key and pub_key:
         return pri_key, pub_key
 
     pri_key, pub_key = generate_ssh_key_pair(key_type=key_type, key_len=key_len)
     if not save_path:
         return pri_key, pub_key
-    return write_key_to_file(pri_key, pub_key, key_type=key_type, save_path=save_path)
+    return write_key_to_file(pri_key, pub_key, key_type=key_type, save_path=save_path, key_name=key_name)
 
 
 def generate_ssh_key_pair(key_type: KeyTypeSupport = KeyTypeSupport.RSA, key_len: int = None) -> Tuple[str, str]:
@@ -205,11 +232,14 @@ def generate_ssh_key_pair(key_type: KeyTypeSupport = KeyTypeSupport.RSA, key_len
     return out.getvalue(), f"{key.get_name()} {key.get_base64()}"
 
 
-def write_key_to_file(pri_key: str, pub_key: str, key_type: KeyTypeSupport, save_path: Path) -> Tuple[str, str]:
-    save_path = save_path.joinpath(".ssh")
+def write_key_to_file(
+    pri_key: str, pub_key: str, key_type: KeyTypeSupport, save_path: Path, key_name: str = None
+) -> Tuple[str, str]:
     save_path.mkdir(parents=True, exist_ok=True, mode=0o700)
-    private_key_file = save_path.joinpath(f"id_{key_type.key.lower()}")
-    public_key_file = save_path.joinpath(f"id_{key_type.key.lower()}.pub")
+    if not key_name:
+        key_name = f"id_{key_type.key.lower()}"
+    private_key_file = save_path.joinpath(key_name)
+    public_key_file = save_path.joinpath(f"{key_name}.pub")
 
     if private_key_file.exists() and public_key_file.exists():
         logger.info(f"ssh key pair exists: {private_key_file}/{public_key_file}, skip generation.")
@@ -224,10 +254,13 @@ def write_key_to_file(pri_key: str, pub_key: str, key_type: KeyTypeSupport, save
     return pri_key, pub_key
 
 
-def try_load_key_from_file(from_path: Path, key_type: KeyTypeSupport, raise_error: bool = False) -> Tuple[str, str]:
-    from_path = from_path.joinpath(".ssh")
-    private_key_file = from_path.joinpath(f"id_{key_type.key.lower()}")
-    public_key_file = from_path.joinpath(f"id_{key_type.key.lower()}.pub")
+def try_load_key_from_file(
+    from_path: Path, key_type: KeyTypeSupport, raise_error: bool = False, key_name: str = None
+) -> Tuple[str, str]:
+    if not key_name:
+        key_name = f"id_{key_type.key.lower()}"
+    private_key_file = from_path.joinpath(key_name)
+    public_key_file = from_path.joinpath(f"{key_name}.pub")
 
     pri_key = pub_key = ""
     if private_key_file.exists():
